@@ -10,12 +10,13 @@
 # ABSTRACT: Fix the configuration of an application
 
 package App::Cme::Command::run ;
-$App::Cme::Command::run::VERSION = '1.017';
+$App::Cme::Command::run::VERSION = '1.018';
 use strict;
 use warnings;
 use 5.10.1;
 use File::HomeDir;
 use Path::Tiny;
+use Config::Model;
 
 use Encode qw(decode_utf8);
 
@@ -28,16 +29,20 @@ sub _set_test_home { $__test_home = shift; }
 
 my $home = $__test_home || File::HomeDir->my_home;
 
-my @script_locs = (
+my @script_paths = map {path($_)} (
     "$home/.cme/scripts",
     "/etc/cme/scripts/",
 );
 
+push @script_paths, path($INC{"Config/Model.pm"})->parent->child("Model/scripts") ;
+
 sub opt_spec {
     my ( $class, $app ) = @_;
     return ( 
         [ "arg=s@"  => "fix only a subset of a configuration tree" ],
-        [ "quiet!"   => "Suppress progress messages" ],
+        [ "quiet!"  => "Suppress progress messages" ],
+        [ "doc!"    => "show documention of script" ],
+        [ "list!"   => "list available scripts" ],
         $class->cme_global_options,
     );
 }
@@ -45,7 +50,7 @@ sub opt_spec {
 sub usage_desc {
   my ($self) = @_;
   my $desc = $self->SUPER::usage_desc; # "%c COMMAND %o"
-  return "$desc [ script ] [ --args foo=12 [ --args bar=13 ]";
+  return "$desc [ script ] [ -args foo=12 [ -args bar=13 ]";
 }
 
 sub description {
@@ -59,6 +64,17 @@ sub execute {
     # see Debian #839593 and perlunicook(1) section X 13
     @$args = map { decode_utf8($_, 1) } @$args;
 
+    if ($opt->{list}) {
+        my @scripts;
+        foreach my $path ( @script_paths ) {
+            next unless $path->is_dir;
+            push @scripts, grep { ! /~$/ } $path->children();
+        }
+        say "Available scripts:";
+        say map {"- ".$_->basename."\n"} @scripts ;
+        return;
+    }
+
     my $script_name = shift @$args;
     my $script;
 
@@ -67,27 +83,52 @@ sub execute {
     }
     else {
         # check script in known locations
-        foreach my $loc ( @script_locs ) {
-            my $path = path($loc);
+        foreach my $path ( @script_paths ) {
             next unless $path->is_dir;
             $script = $path->child($script_name);
             last if $script->is_file;
         }
     }
 
+    die "Error: cannot find script $script_name\n" unless $script->is_file;
+
     my $content = $script->slurp_utf8;
 
-    # tweak variables
+    # parse variables passed on command line
     my %fill_h = map { split '=',$_,2; } @{ $opt->{arg} };
-    $content =~ s~ (?<!\\) \$(\w+) ~ $fill_h{$1} // '$'.$1 ~xeg;
-    $content =~ s!\\\$!\$!g;
 
+    if ($content =~ m/^#!/ or $content =~ /^use/m) {
+        splice @ARGV, 0,2; # remove 'run script' arguments
+        eval $script->slurp_utf8;
+        die "Error in script $script_name: $@\n" if $@;
+        return;
+    }
+
+    my %var;
+
+    # find if all variables are accounted for
+    my @missing ;
+
+    # replace variables with command arguments or eval'ed variables or env variables
+    my $replace_var = sub {
+        # change $var but not \$var
+        $_[0] =~ s~ (?<!\\) \$(\w+) ~ $fill_h{$1} // $var{$1} // $ENV{$1} // '$'.$1~xeg;
+
+        # register vars without replacements
+        push @missing, ($_[0] =~ m~ (?<!\\) \$(\w+) ~xg);
+
+        # now change \$var in $var
+        $_[0] =~ s!\\\$!\$!g;
+    };
+
+    my @lines =  split /\n/,$content;
     my @load;
+    my @doc;
     my $commit_msg ;
     my $line_nb = 0;
 
     # check content, store app
-    foreach my $line ( split /\n/,$content ) {
+    foreach my $line ( @lines ) {
         $line_nb++;
         $line =~ s/#.*//; # remove comments
         $line =~ s/^\s+//;
@@ -96,9 +137,18 @@ sub execute {
 
         next unless $key ; # empty line
 
+        $replace_var->($value) unless $key eq 'var';
+
         if ($key =~ /^app/) {
             unshift @$args, $value;
         }
+        elsif ($key eq 'var') {
+            eval ($value) ;
+            die "Error in var specification line $line_nb: $@\n" if $@;
+        }
+        elsif ($key eq 'doc') {
+            push @doc, $value;
+        }
         elsif ($key eq 'load') {
             push @load, $value;
         }
@@ -110,6 +160,16 @@ sub execute {
         }
     }
 
+    if ($opt->doc) {
+        say join "\n", @doc;
+        return;
+    }
+
+    if (@missing) {
+        die "Error: Missing variables @missing in command arguments for script $script\n"
+            ."Please use option '-arg ".join(',', map { "$_=xxx"} @missing)."'\n";
+    }
+
     $self->process_args($opt, $args);
 
     # check if workspace and index are clean
@@ -129,7 +189,6 @@ sub execute {
     if ($commit_msg) {
         system(qw/git commit -a -m/, $commit_msg);
     }
-
 }
 
 1;
@@ -146,11 +205,12 @@ App::Cme::Command::run - Fix the configuration of an application
 
 =head1 VERSION
 
-version 1.017
+version 1.018
 
 =head1 SYNOPSIS
 
  $ cat ~/.cme/scripts/remove-mia
+ doc: remove mia from Uploders. Require mia parameter
  # declare app to configure
  app: dpkg
  # specify one or more instructions
@@ -158,11 +218,47 @@ version 1.017
  # commit the modifications with a message (git only)
  commit: remove MIA dev $mia
 
- $ cme run remove-mia --arg mia=longgone@d3bian.org
+ $ cme run remove-mia -arg mia=longgone@d3bian.org
+
+ # cme run can also use environment variables
+ $ cat ~/.cme/scripts/add-me-to-uploaders
+ app: dpkg-control
+ load: source Uploaders:.push("$DEBFULLNAME <$DEBEMAIL>")
+
+ $ cme run add-me-to-uploaders
+ Reading package lists... Done
+ Building dependency tree
+ Reading state information... Done
+ Changes applied to dpkg-control configuration:
+ - source Uploaders:3: '<undef>' -> 'Dominique Dumont <dod@debian.org>'
+
+ # show the script documentation
+ $ cme run remove-mia -doc
+ remove mia from Uploders. require mia parameter
+
+ # list scripts
+ $ cme run -list
+ Available scripts:
+ - update-copyright
+ - add-me-to-uploaders
 
 =head1 DESCRIPTION
 
-Run a script using cme DSL (Design specfic language).
+Run a script written with cme DSL (Design specific language) or in
+plain Perl.
+
+A script passed by name is searched in C<~/.cme/scripts>,
+C</etc/cme/scripts> or C</usr/share/perl5/Config/Model/scripts>.
+E.g. with C<cme run foo>, C<cme> loads either C<~/.cme/scripts/foo>,
+C</etc/cme/scripts/foo> or
+C</usr/share/perl5/Config/Model/scripts/foo>
+
+No search is done if the script is passed with a path
+(e.g. C<cme run ./foo>)
+
+C<cme run> can also run plain Perl script. This is syntactic sugar to
+avoid polluting global namespace, i.e. there's no need to store a
+script using L<cme function|Config::Model/cme> in C</usr/local/bin/>.
 
 When run, this script:
 
@@ -170,27 +266,23 @@ When run, this script:
 
 =item *
 
-Open the configuration file of C<app>
+opens the configuration file of C<app>
 
 =item *
 
-Apply the modifications specified with C<load> instructions
+applies the modifications specified with C<load> instructions
 
 =item *
 
-Save the configuration files.
+save the configuration files
 
 =item *
 
-Commit the result if C<commit> is specified.
+commits the result if C<commit> is specified.
 
 =back
 
-A script passed by name is searched in C<~/.cme/scripts> and in
-C</etc/cme/scripts>. (e.g C<cme run foo>)
-
-No search is done if the script is passed with a path 
-(e.g. C<cme run ./foo>)
+See L<App::Cme::Command::run> for details.
 
 =head1 Syntax
 
@@ -198,7 +290,7 @@ The script accepts instructions in the form:
 
  key: value
 
-The script accept 3 instructions:
+The script accepts the following instructions:
 
 =over
 
@@ -208,6 +300,13 @@ Specify the target application. Must be one of the application listed
 by C<cme list> command. Mandatory. Only one C<app> instruction is
 allowed.
 
+=item var
+
+Use Perl code to specify variables usable in this script. The Perl
+code must store data in C<%var> hash. For instance:
+
+    perl: my @l = localtime; $var{year} =  $l[5]+1900;
+
 =item load
 
 Specify the modifications to apply using a string as specified in
@@ -215,18 +314,19 @@ L<Config::Model::Loader>
 
 =item commit
 
-Specify that the change must be commited with the passed commit
+Specify that the change must be committed with the passed commit
 message. When this option is used, C<cme> raises an error if used on a
 non-clean workspace. This option works only with L<git>.
 
 =back
 
-All instructions can use variables like C<$stuff> and specify
-the value with C<cme> options
+All instructions can use variables like C<$stuff> whose value can be
+specified with C<-arg> options, with a Perl variable (from C<var:>
+section explained above) or with an environment variable:
 
 For instance:
 
-  cme run --arg var1=foo --arg var2=bar
+  cme run -arg var1=foo -arg var2=bar
 
 transforms the instruction:
 
@@ -238,9 +338,18 @@ in
 
 =head1 Options
 
+=head2 list
+
+List available scripts and exits.
+
 =head2 arg
 
-Arguments for the cme scripts which are used to substiture variables.
+Arguments for the cme scripts which are used to substitute variables.
+
+=head2 doc
+
+Show the script documentation. (Note that C<--help> options show the
+documentation of C<cme run> command)
 
 =head1 Common options
 
@@ -254,7 +363,7 @@ See L<cme/"Global Options">.
  app: dpkg-copyright
  load: Files:~ Copyright=~"s/2016,?\s+$name/2017, $name/g"
  commit: updated copyright year of $name
- $ cme run update-copyright --arg "name=Dominique Dumont"
+ $ cme run update-copyright -arg "name=Dominique Dumont"
  cme: using Dpkg::Copyright model
  Changes applied to dpkg-copyright configuration:
  - Files:"*" Copyright: '2005-2016, Dominique Dumont <dod@debian.org>' -> '2005-2017, Dominique Dumont <dod@debian.org>'
