@@ -10,7 +10,7 @@
 #ABSTRACT: Common methods for App::Cme
 
 package App::Cme::Common;
-$App::Cme::Common::VERSION = '1.017';
+$App::Cme::Common::VERSION = '1.018';
 use strict;
 use warnings;
 use 5.10.1;
@@ -31,7 +31,8 @@ sub cme_global_options {
       [ "try-app-as-model!"  => "try to load a model using directly the application name "
                               . "specified as 3rd parameter on the command line"],
       [ "dev!"               => "test a model under development"],
-      [ "force-load!"        => "Load file even if error are found in data. Bad data are discarded"],
+      [ "save!"              => "Force a save even if no change was done" ],
+      [ "force-load!"        => "Load file even if error are found in data. Bad data are discarded (imply save)"],
       [ "create!"            => "start from scratch."],
       [ "root-dir=s"         => "Change root directory. Mostly used for test"],
       [ "file=s"             => "Specify a target file"],
@@ -228,7 +229,7 @@ App::Cme::Common - Common methods for App::Cme
 
 =head1 VERSION
 
-version 1.017
+version 1.018
 
 =head1 SYNOPSIS
 
